#!/bin/bash

# Allow iteration per-line.
IFS=$'\n'

# Style variables.
STYLE_BOLD=$(tput bold)
STYLE_UNDERLINE=$(tput smul)
STYLE_NORMAL=$(tput sgr0)

# Initialize task switches.
INSTALL_ANCHOR=0
RESET_STORAGE=0
REALLY_RESET_STORAGE=0
CHOICE_VALID=0
APP_MODE=0
STDOUT=/dev/stdout
APPOUT=/dev/null

# Begin validating which tasks to perform.
CHOICE_VALID=0

while [[ "$#" -ne 0 ]]
do 
    case $1 in 
        -h|-help|--help)
            cat <<usageInfo
Toon Boom Licensing Service Repair Utility v0.5

You can run this script without parameters. It simply will prompt you for 
confirmation before re-installing the activation service.

    -f|--fix                Re-install the anchor service and fix the license 
                            storage permissions.   
    -h|--help               Display this help information.
    --hard                  Deletes local licenses, re-install the anchor 
                            service and fix the license storage permissions.   
    --delete                Deletes local licenses. 
    --app-mode <log file>   Outputs information in a format for an application 
                            message box to the standard output, and the extended
                            information to the log file in the specified path. 

usageInfo
            exit 0
        ;;
        -f|-fix|--fix)
            INSTALL_ANCHOR=1
            CHOICE_VALID=1
            shift 
        ;;
        -hard|--hard)
            INSTALL_ANCHOR=1
            RESET_STORAGE=1
            CHOICE_VALID=1
            shift 
        ;;
        -delete|--delete)
            RESET_STORAGE=1
            CHOICE_VALID=1
            shift 
        ;;
        -app-mode|--app-mode)
            APP_MODE=1
            APPOUT=/dev/stdout            
            shift 
            STDOUT=$1
            shift             
        ;;
    esac
done 
# Check for administrator privileges. 
PARAMETERS_BAK=("$@")
if [[ $(id -u) -ne 0 ]]
then
    if [[ $APP_MODE == 0 ]]
    then 
        # If system has sudo, prompt to relaunch script with sudo. Otherwise, quit with error.
        SUDO_BIN="$(which sudo 2>/dev/null)"
        if [[ -z "$SUDO_BIN" ]]
        then
                echo "This script requires administrator privileges. Please relaunch it as the root user."
                exit 0
        else
                echo "This utility requires administrator privileges. To authorize it to make changes
to your computer, please type in your account password and press ${STYLE_BOLD}ENTER${STYLE_NORMAL}."
                $SUDO_BIN -k $0 "${PARAMETERS_BAK[@]}"
                exit 0
        fi
    else 
        echo "Error: This script requires administrator privileges."
        exit 1
    fi 
fi

# Set working directory to script directory.
cd "$(dirname ${BASH_SOURCE[0]})"

if [[ $CHOICE_VALID == 1 ]]
then 
    # Welcome message. 
    echo "Toon Boom Licensing Service Repair Utility v0.5" > $STDOUT
elif [[ $CHOICE_VALID == 0 && $APP_MODE == 1 ]]
then
    echo "Error: No action parameters specified."
    exit 1
else
    # Launch Interactive mode if no parameters were used. 
    # Welcome message. 
    echo "Welcome to the ${STYLE_UNDERLINE}Toon Boom Licensing Service Repair Utility v0.5${STYLE_NORMAL}."
    echo

    # Repeat prompt until a valid choice is provided. 
    while [[ $CHOICE_VALID == 0 ]]
    do
        read -p "Do you want to re-install the license validation service for your Toon Boom 
applications? ${STYLE_BOLD}[yes/no]${STYLE_NORMAL} " CHOICE

        # Convert answer to lowercase to make it case insensitive. 
        CHOICE=$(echo "$CHOICE" | tr '[:upper:]' '[:lower:]')
        
        case $CHOICE in 
            y|yes)
                INSTALL_ANCHOR=1
                CHOICE_VALID=1
            ;;
            n|no)
                CHOICE_VALID=1
            ;;
        esac
    done
fi 

# Remove licenses.
if [[ $RESET_STORAGE == 1 ]]
then 
    CONFIRM_DELETE_VALID=0 
    
    if [[ $APP_MODE == 1 ]] 
    then
        CONFIRM_DELETE_VALID=1
    fi
    
    while [[ $CONFIRM_DELETE_VALID == 0 ]]
    do 
        read -p "WARNING: This will delete your local licenses. If you have any licenses 
currently activated on computer, you will need to contact support to be able to
activate them again. Do you want to continue? ${STYLE_BOLD}[yes/no]${STYLE_NORMAL} " CONFIRM_DELETE
    
        # Convert answer to lowercase to make it case insensitive. 
        CONFIRM_DELETE=$(echo "$CONFIRM_DELETE" | tr '[:upper:]' '[:lower:]')
        
        case $CONFIRM_DELETE in
            y|yes)
                CONFIRM_DELETE_VALID=1
                REALLY_RESET_STORAGE=1

            ;;
            n|no)
                CONFIRM_DELETE_VALID=1
                REALLY_RESET_STORAGE=0
            ;;
        esac 
    done 
fi

if [[ $REALLY_RESET_STORAGE == 1 ]]
then 
    echo "Deleting local licenses..."
    rm -fv /Library/Preferences/FLEXnet\ Publisher/FLEXnet/toonboom_*
fi

# Re-install anchor services. 
if [[ $INSTALL_ANCHOR == 1 ]]
then 
    echo "The Toon Boom Licensing Service Repair Utility did the following:" > $APPOUT
    echo >> $APPOUT

    cd ./flexlm_scripts
    # Iterate through each directory bundled with this utility. 
    for INSTALL_FNP_DIR in $(find ./ -type d -mindepth 1 -maxdepth 1 | sort -V)
    do 
        # Check if directory contains an install_fnp.sh script. 
        if [[ -x "$INSTALL_FNP_DIR/install_fnp.sh" ]]
        then 
            INSTALL_FNP_VER=$(basename $INSTALL_FNP_DIR)
            THIS_FNP_WAS_UNINSTALLED=0
            # Versions in the utility are labeled in the v#.# format, whereas those installed are labeled in the #.#.# format.
            # So we have to process the string first.
            # Break the version string into major, minor and patch number, excluding the v and excluding any extra number.
            # This handles cases where the version is a single number or two numbers separated by a period or more.
            if [[ $INSTALL_FNP_VER =~ ^v([[:digit:]]+)\.?([[:digit:]]+)?\.?([[:digit:]]+)?.*$ ]]
            then 
                MAJOR_VER=${BASH_REMATCH[1]}
                [[ -n ${BASH_REMATCH[2]} ]] && MINOR_VER=${BASH_REMATCH[2]} || MINOR_VER=0
                [[ -n ${BASH_REMATCH[3]} ]] && PATCH_VER=${BASH_REMATCH[3]} || PATCH_VER=0            
                INSTALL_FNP_VER_CLEAN="${MAJOR_VER}.${MINOR_VER}.${PATCH_VER}"
                
                if [[ -d "/Library/Application Support/FLEXnet Publisher/Service/${INSTALL_FNP_VER_CLEAN}" ]]
                then             
                    echo "Removing old anchor service version $INSTALL_FNP_VER..." >> $STDOUT
                    rm -frv "/Library/Application Support/FLEXnet Publisher/Service/${INSTALL_FNP_VER_CLEAN}" >> $STDOUT
                    [[ $? == 0 ]] && THIS_FNP_WAS_UNINSTALLED=1
                fi
            fi
            
            # Run the install script for this version. 
            echo >> $STDOUT
            echo "Installing anchor service version $INSTALL_FNP_VER..." >> $STDOUT    
            cd "${INSTALL_FNP_DIR}" 
            ./install_fnp.sh >> $STDOUT 
            if [[ $? == 0 ]] 
            then
                if [[ $THIS_FNP_WAS_UNINSTALLED == 1 ]]
                then 
                    echo " • Uninstalled and re-installed license service version $INSTALL_FNP_VER." > $APPOUT
                else
                    echo " • Installed license service version $INSTALL_FNP_VER." > $APPOUT
                fi 
            fi 
            cd ..        
        fi 
    done 
fi

if [[ $REALLY_RESET_STORAGE == 1 ]]
then 
    echo 
    echo "${STYLE_BOLD}WARNING: Your local licenses were deleted. Before you can activate them, 
you must contact support and ask them to return your licenses so that they can 
be activated again. You can contact support at ${STYLE_UNDERLINE}support@toonboom.com${STYLE_NORMAL}${STYLE_BOLD}.${STYLE_NORMAL}" >> $STDOUT
    echo >> $STDOUT
elif [[ $REALLY_RESET_STORAGE == 0 && $INSTALL_ANCHOR == 1 ]]
then
    echo 
    echo "${STYLE_BOLD}You can now try activating your product again. If you are still getting 
Error -1, please contact support at ${STYLE_UNDERLINE}support@toonboom.com${STYLE_NORMAL}${STYLE_BOLD}.${STYLE_NORMAL}" >> $STDOUT
    echo "You can now try activating your product again. 

If you are still getting Error -1, please contact support at support@toonboom.com." >> $APPOUT
    echo >> $STDOUT
fi 